using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        FormatDef format;
        private void Form1_Load(object sender, EventArgs e)
        {
            Stream formatRes = Resource.GetStream("QiHe.RealMedia.FormatDef.xml");
            format = XmlData<FormatDef>.Load(formatRes);
            format.Group();
        }

        MediaFile media;
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string file = FileSelector.BrowseFile(FileType.All);
            if (file != null)
            {
                if (media != null)
                {
                    media.FileStream.Close();
                }
                bool hasError;
                StatusText = "Chargement...";
                media = Reader.Load(file, format, out hasError);
                ShowMediaFile(media);
                StatusText = "Charg";
                if (hasError)
                {
                    MessageBox.Show("Il y a des erreurs de dcodage, le fichier peut ne pas tre enregistr jusqu' ce que vous les ayez corrig.");
                }
            }
        }

        private void ShowMediaFile(MediaFile media)
        {
            TreeNode node = new TreeNode(media.FileName);
            node.Tag = media;
            foreach (Chunk chunk in media.Chunks)
            {
                TreeNode chunknode = CreateTreeNode(chunk.Record);
                chunknode.ContextMenu = new ContextMenu(
                    new MenuItem[]{
                        new MenuItem("Supprimer", delegate(object sender, EventArgs e)
                        {
                            media.Chunks.Remove(chunknode.ContextMenu.Tag as Chunk);
                            Writer.CalculateSize(media, format);
                            ShowMediaFile(media);
                        })
                    });
                chunknode.ContextMenu.Tag = chunk;
                node.Nodes.Add(chunknode);
            }
            treeView1.Nodes.Clear();
            treeView1.Nodes.Add(node);
            node.Expand();
            treeView1.SelectedNode = node;
        }

        TreeNode CreateTreeNode(Record record)
        {
            if (record == null) return new TreeNode("null");
            TreeNode node = new TreeNode(record.Name);
            node.Tag = record;
            foreach (KeyValuePair<string, object> field in record.Fields)
            {
                if (field.Value is Record)
                {
                    node.Nodes.Add(CreateTreeNode(field.Value as Record));
                }
                else if (field.Value is IEnumerable<Record>)
                {
                    TreeNode subnode = new TreeNode(field.Key);
                    subnode.Tag = field.Value;
                    node.Nodes.Add(subnode);
                }
            }
            return node;
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            listView1.Items.Clear();
            TreeNode node = treeView1.SelectedNode;
            if (node.Tag is MediaFile)
            {
                SetTitle("ID Section", "Position", "Taille");
                MediaFile media = node.Tag as MediaFile;
                foreach (Chunk chunk in media.Chunks)
                {
                    string[] items = new string[] {
                        chunk.Name,
                        chunk.Offset.ToString(),
                        chunk.Size.ToString() };
                    ListViewItem item = new ListViewItem(items);
                    listView1.Items.Add(item);
                }
            }
            else if (node.Tag is Record)
            {
                SetTitle("Nom du Champ", "Valeur");
                Record record = node.Tag as Record;
                foreach (KeyValuePair<string, object> field in record.Fields)
                {
                    string[] items = new string[] { field.Key, field.Value.ToString() };
                    ListViewItem item = new ListViewItem(items);
                    listView1.Items.Add(item);
                }
            }
            else if (node.Tag is List<Record>)
            {
                List<Record> records = node.Tag as List<Record>;
                if (records.Count > 0)
                {
                    string[] keys = new List<string>(records[0].Fields.Keys).ToArray();
                    SetTitle(keys);
                    int count = Math.Min(100, records.Count);
                    for (int n = 0; n < count; n++)
                    {
                        Record record = records[n];
                        AddRecordToListView(keys, record);
                    }
                }
            }
            else if (node.Tag is RecordCollection)
            {
                RecordCollection records = node.Tag as RecordCollection;
                records.Reset();
                Record first_record = records.Read();
                string[] keys = new List<string>(first_record.Fields.Keys).ToArray();
                SetTitle(keys);
                AddRecordToListView(keys, first_record);
                int count = Math.Min(100, records.Count);
                for (int n = 1; n < count; n++)
                {
                    Record record = records.Read();
                    AddRecordToListView(keys, record);
                    if (record.HasError)
                    {
                        break;
                    }
                }
            }
        }

        private void AddRecordToListView(string[] keys, Record record)
        {
            string[] items = new string[record.Fields.Count];
            for (int i = 0; i < items.Length; i++)
            {
                items[i] = record.Fields[keys[i]].ToString();
            }
            ListViewItem item = new ListViewItem(items);
            listView1.Items.Add(item);
        }

        void SetTitle(params string[] headers)
        {
            if (listView1.Columns.Count == headers.Length)
            {
                for (int i = 0; i < headers.Length; i++)
                {
                    listView1.Columns[i].Text = headers[i];
                }
            }
            else
            {
                int width = listView1.Width / headers.Length;
                ColumnHeader[] colheaders = new ColumnHeader[headers.Length];
                for (int i = 0; i < headers.Length; i++)
                {
                    ColumnHeader colheader = new ColumnHeader();
                    colheader.Text = headers[i];
                    colheader.Width = width;
                    colheaders[i] = colheader;
                }
                listView1.Columns.Clear();
                listView1.Columns.AddRange(colheaders);
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MediaFile media = treeView1.Nodes[0].Tag as MediaFile;
            SaveMediaFile(media);
        }

        private void SaveMediaFile(MediaFile media)
        {
            string file = FileSelector.BrowseFileForSave(FileType.All);
            if (file != null)
            {
                StatusText = "Enregistrement...";
                try
                {
                    Writer.Save(file, media, format);
                    StatusText = "Enregistr";
                }
                catch (Exception error)
                {
                    MessageBox.Show(error.Message);
                    StatusText = "Echec de l'enregistrement";
                }
            }
        }

        private void rebuildToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StatusText = "Cration de l'index...";
            MediaFile media = treeView1.Nodes[0].Tag as MediaFile;
            Process.RebuildIndex(media);
            Writer.CalculateSize(media, format);
            ShowMediaFile(media);
            StatusText = "Termin";
        }

        private void checkErrorsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StatusText = "Recherche d'erreurs...";
            MediaFile media = treeView1.Nodes[0].Tag as MediaFile;
            List<string> errors = Process.CheckErrors(media, format);
            StatusText = String.Format("{0} erreurs trouves", errors.Count);
            if (errors.Count > 0)
            {
                ShowErrors(errors, media.FileName);
            }
        }

        HighTimer timer = new HighTimer();
        private void readToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StatusText = "Rcupration de tous les paquets mdia...";
            MediaFile media = treeView1.Nodes[0].Tag as MediaFile;
            timer.Start();
            PacketRecover recover = new PacketRecover(media, format);
            recover.RecoverPackets();
            timer.Stop();
            ShowLogInfo(recover.LogInfo);
            Writer.CalculateSize(media, format);
            ShowMediaFile(media);
            StatusText = String.Format("Termin en {0:F4} secondes.", timer.Duration);
        }

        private void ShowErrors(List<string> errors, string filename)
        {
            string file = WinApp.AppPath + "errors.txt";
            StreamWriter writer = new StreamWriter(file);
            writer.WriteLine(String.Format("{0} erreurs ont t trouves dans {1}.", errors.Count, filename));
            foreach (string error in errors)
            {
                writer.WriteLine(error);
            }
            writer.Close();
            System.Diagnostics.Process.Start(file);
        }

        public void ShowLogInfo(List<string> loginfo)
        {
            string file = WinApp.AppPath + "fix-log.txt";
            StreamWriter writer = new StreamWriter(file);
            foreach (string log in loginfo)
            {
                writer.WriteLine(log);
            }
            writer.Close();
            System.Diagnostics.Process.Start(file);
        }

        string StatusText
        {
            set
            {
                toolStripStatusLabel1.Text = value;
                Application.DoEvents();
            }
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CutterDialog cutter = new CutterDialog();
            if (cutter.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    int start = cutter.StartTime;
                    int end = cutter.EndTime;
                    MediaFile media = treeView1.Nodes[0].Tag as MediaFile;
                    MediaFile portion = Process.CutMedia(media, start, end);
                    Writer.CalculateSize(portion, format);
                    Process.RebuildIndex(portion);
                    Writer.CalculateSize(portion, format);
                    SaveMediaFile(portion);
                }
                catch (Exception error)
                {
                    MessageBox.Show(error.Message);
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}