using System;
using System.Collections.Generic;
using System.Text;

namespace QiHe.RealMedia
{
    public partial class Process
    {
        public static List<string> CheckErrors(MediaFile media, FormatDef format)
        {
            List<string> errors = new List<string>();
            CheckLinkErrors(media, errors);
            CheckSizeErrors(media, format, errors);
            return errors;
        }

        private static void CheckSizeErrors(MediaFile media, FormatDef format, List<string> errors)
        {
            foreach (Chunk chunk in media.Chunks)
            {
                RecordDef recordDef = format[chunk.Name];
                if (recordDef != null)
                {
                    int datasize = recordDef.CalculateSize(chunk.Record, format);
                    if (chunk.Size != (uint)(datasize + 8))
                    {
                        errors.Add(String.Format(
                            "Pour la section {0}, la taille indique de {1} et la taille calcule de {2} ne correspondent pas.",
                            chunk.Name, chunk.Size, datasize + 8));
                    }
                }
                else
                {
                    errors.Add("Type de section inconnu: " + chunk.Name);
                }
            }
        }

        private static void CheckLinkErrors(MediaFile media, List<string> errors)
        {
            Chunk prop = null;
            Chunk data = null;
            Chunk index = null;
            foreach (Chunk chunk in media.Chunks)
            {
                if (chunk.Name == "PROP")
                {
                    prop = chunk;
                }
                else if (chunk.Name == "DATA")
                {
                    if (data == null)
                    {
                        if ((uint)prop["data_offset"] != (uint)chunk.Offset)
                        {
                            errors.Add(String.Format(
                                "PROP.data_offset {0} et DATA.Offset {1} ne correspondent pas.",
                                prop["data_offset"], chunk.Offset
                                ));
                        }
                    }
                    else
                    {
                        if ((uint)data["next_data_header"] != (uint)chunk.Offset)
                        {
                            errors.Add(String.Format(
                                "DATA.next_data_header {0} et DATA.Offset {1} ne correspondent pas.",
                                data["next_data_header"], chunk.Offset
                                ));
                        }
                    }
                    data = chunk;
                }
                else if (chunk.Name == "INDX")
                {
                    if (index == null)
                    {
                        if ((uint)prop["index_offset"] != (uint)chunk.Offset)
                        {
                            errors.Add(String.Format(
                                "PROP.index_offset {0} et INDX.Offset {1} ne correspondent pas.",
                                prop["index_offset"], chunk.Offset
                                ));
                        }
                    }
                    else
                    {
                        if ((uint)index["next_index_header"] != (uint)chunk.Offset)
                        {
                            errors.Add(String.Format(
                                "INDX.next_index_header {0} et INDX.Offset {1} ne correspondent pas.",
                                index["next_index_header"], chunk.Offset
                                ));
                        }
                    }
                    index = chunk;
                }
            }
            if (data == null)
            {
                errors.Add("Section DATA introuvable");
            }
            if (index == null)
            {
                errors.Add("Section INDX introuvable");
            }
        }
    }
}
